@@ -41,8 +41,9 @@
 #include <QTabWidget>
 #include <QFile>
 #include <QStringList>
+#include <QUrl>
 
-struct KNotification::Private {
+struct Q_DECL_HIDDEN KNotification::Private {
     QString eventId;
     int id;
     int ref;
@@ -51,16 +52,19 @@ struct KNotification::Private {
     QString title;
     QString text;
     QString iconName;
+    QString defaultAction;
     QStringList actions;
     QPixmap pixmap;
     ContextList contexts;
     NotificationFlags flags;
     QString componentName;
+    QList<QUrl> urls;
+    QVariantMap hints;
 
     QTimer updateTimer;
     bool needUpdate;
 
-    Private() : id(0), ref(0), widget(0l), needUpdate(false) {}
+    Private() : id(-1), ref(0), widget(nullptr), needUpdate(false) {}
     /**
      * recursive function that raise the widget. @p w
      *
@@ -75,7 +79,7 @@ KNotification::KNotification(const QString &eventId, QWidget *parent, const Noti
     d->eventId = eventId;
     d->flags = flags;
     setWidget(parent);
-    connect(&d->updateTimer, SIGNAL(timeout()), this, SLOT(update()));
+    connect(&d->updateTimer, &QTimer::timeout, this, &KNotification::update);
     d->updateTimer.setSingleShot(true);
     d->updateTimer.setInterval(100);
 }
@@ -89,7 +93,7 @@ KNotification::KNotification(
 {
     d->eventId = eventId;
     d->flags = flags;
-    connect(&d->updateTimer, SIGNAL(timeout()), this, SLOT(update()));
+    connect(&d->updateTimer, &QTimer::timeout, this, &KNotification::update);
     d->updateTimer.setSingleShot(true);
     d->updateTimer.setInterval(100);
     d->widget = nullptr;
@@ -97,7 +101,7 @@ KNotification::KNotification(
 
 KNotification::~KNotification()
 {
-    if (d->id > 0) {
+    if (d->id >= 0) {
         KNotificationManager::self()->close(d->id);
     }
     delete d;
@@ -140,7 +144,7 @@ void KNotification::setTitle(const QString &title)
 
     d->needUpdate = true;
     d->title = title;
-    if (d->id > 0) {
+    if (d->id >= 0) {
         d->updateTimer.start();
     }
 }
@@ -153,7 +157,7 @@ void KNotification::setText(const QString &text)
 
     d->needUpdate = true;
     d->text = text;
-    if (d->id > 0) {
+    if (d->id >= 0) {
         d->updateTimer.start();
     }
 }
@@ -166,7 +170,7 @@ void KNotification::setIconName(const QString &icon)
 
     d->needUpdate = true;
     d->iconName = icon;
-    if (d->id > 0) {
+    if (d->id >= 0) {
         d->updateTimer.start();
     }
 }
@@ -185,7 +189,7 @@ void KNotification::setPixmap(const QPixmap &pix)
 {
     d->needUpdate = true;
     d->pixmap = pix;
-    if (d->id > 0) {
+    if (d->id >= 0) {
         d->updateTimer.start();
     }
 }
@@ -203,11 +207,29 @@ void KNotification::setActions(const QStringList &as)
 
     d->needUpdate = true;
     d->actions = as;
-    if (d->id > 0) {
+    if (d->id >= 0) {
         d->updateTimer.start();
     }
 }
 
+void KNotification::setDefaultAction(const QString &defaultAction)
+{
+    if (defaultAction == d->defaultAction) {
+        return;
+    }
+
+    d->needUpdate = true;
+    d->defaultAction = defaultAction;
+    if (d->id >= 0) {
+        d->updateTimer.start();
+    }
+}
+
+QString KNotification::defaultAction() const
+{
+    return d->defaultAction;
+}
+
 KNotification::ContextList KNotification::contexts() const
 {
     return d->contexts;
@@ -243,6 +265,16 @@ void KNotification::setComponentName(const QString &c)
     d->componentName = c;
 }
 
+QList<QUrl> KNotification::urls() const
+{
+    return d->urls;
+}
+
+void KNotification::setUrls(const QList<QUrl> &urls)
+{
+    d->urls = urls;
+}
+
 void KNotification::activate(unsigned int action)
 {
     switch (action) {
@@ -264,8 +296,6 @@ void KNotification::activate(unsigned int action)
     // which will deref() the KNotification object, which will result
     // in closing the notification
     emit activated(action);
-
-    d->id = -1;
 }
 
 void KNotification::close()
@@ -279,7 +309,6 @@ void KNotification::close()
         emit closed();
         deleteLater();
     }
-
 }
 
 void KNotification::raiseWidget()
@@ -296,10 +325,9 @@ void KNotification::Private::raiseWidget(QWidget *w)
     //TODO  this function is far from finished.
     if (w->isTopLevel()) {
         w->raise();
-#if defined(Q_OS_MAC)
-        w->activateWindow();
-#else
         KWindowSystem::activateWindow(w->winId());
+#if defined(Q_OS_MACOS)
+        w->activateWindow();
 #endif
     } else {
         QWidget *pw = w->parentWidget();
@@ -311,6 +339,15 @@ void KNotification::Private::raiseWidget(QWidget *w)
     }
 }
 
+static QString defaultComponentName()
+{
+#ifndef Q_OS_ANDROID
+    return QStringLiteral("plasma_workspace");
+#else
+    return QStringLiteral("android_defaults");
+#endif
+}
+
 KNotification *KNotification::event(const QString &eventid, const QString &title, const QString &text,
                                     const QPixmap &pixmap, QWidget *widget, const NotificationFlags &flags, const QString &componentName)
 {
@@ -318,9 +355,9 @@ KNotification *KNotification::event(const QString &eventid, const QString &title
     notify->setTitle(title);
     notify->setText(text);
     notify->setPixmap(pixmap);
-    notify->setComponentName(flags & DefaultEvent ? QStringLiteral("plasma_workspace") : componentName);
+    notify->setComponentName((flags & DefaultEvent) ? defaultComponentName() : componentName);
 
-    QTimer::singleShot(0, notify, SLOT(sendEvent()));
+    QTimer::singleShot(0, notify, &KNotification::sendEvent);
 
     return notify;
 }
@@ -352,9 +389,9 @@ KNotification *KNotification::event(const QString &eventid, const QString &title
     notify->setTitle(title);
     notify->setText(text);
     notify->setIconName(iconName);
-    notify->setComponentName(flags & DefaultEvent ? QStringLiteral("plasma_workspace") : componentName);
+    notify->setComponentName((flags & DefaultEvent) ? defaultComponentName() : componentName);
 
-    QTimer::singleShot(0, notify, SLOT(sendEvent()));
+    QTimer::singleShot(0, notify, &KNotification::sendEvent);
 
     return notify;
 }
@@ -376,9 +413,9 @@ void KNotification::ref()
 {
     d->ref++;
 }
-
 void KNotification::deref()
 {
+    Q_ASSERT(d->ref > 0);
     d->ref--;
     if (d->ref == 0) {
         d->id = -1;
@@ -394,9 +431,9 @@ void KNotification::beep(const QString &reason, QWidget *widget)
 void KNotification::sendEvent()
 {
     d->needUpdate = false;
-    if (d->id == 0) {
+    if (d->id == -1) {
         d->id = KNotificationManager::self()->notify(this);
-    } else if (d->id > 0) {
+    } else if (d->id >= 0) {
         KNotificationManager::self()->reemit(this);
     }
 }
@@ -414,7 +451,7 @@ QString KNotification::appName() const
     QString appname;
 
     if (d->flags & DefaultEvent) {
-        appname = QStringLiteral("plasma_workspace");
+        appname = defaultComponentName();
     } else if (!d->componentName.isEmpty()) {
         appname = d->componentName;
     } else {
@@ -436,7 +473,7 @@ bool KNotification::eventFilter(QObject *watched, QEvent *event)
     if (watched == d->widget) {
         if (event->type() == QEvent::WindowActivate) {
             if (d->flags &  CloseWhenWidgetActivated) {
-                QTimer::singleShot(500, this, SLOT(close()));
+                QTimer::singleShot(500, this, &KNotification::close);
             }
         }
     }
@@ -487,3 +524,21 @@ QString KNotification::standardEventToIconName(KNotification::StandardEvent even
     return iconName;
 }
 
+void KNotification::setHint(const QString &hint, const QVariant &value)
+{
+    if (value == d->hints.value(hint)) {
+        return;
+    }
+
+    d->needUpdate = true;
+    d->hints[hint] = value;
+    if (d->id >= 0) {
+        d->updateTimer.start();
+    }
+}
+
+QVariantMap KNotification::hints() const
+{
+    return d->hints;
+}
+
