@@ -38,6 +38,10 @@ static KSharedConfig::Ptr retrieve_from_cache(const QString &filename, QStandard
     }
 
     KSharedConfig::Ptr m = KSharedConfig::openConfig(filename, KConfig::NoGlobals, type);
+    // also search for event config files in qrc resources
+    if (type == QStandardPaths::GenericDataLocation) {
+        m->addConfigSources({QStringLiteral(":/") + filename});
+    }
     cache.insert(filename, new KSharedConfig::Ptr(m));
 
     return m;
@@ -46,7 +50,8 @@ static KSharedConfig::Ptr retrieve_from_cache(const QString &filename, QStandard
 void KNotifyConfig::reparseConfiguration()
 {
     QCache<QString, KSharedConfig::Ptr> &cache = *static_cache;
-    Q_FOREACH (const QString &filename, cache.keys()) {
+    const auto listFiles =  cache.keys();
+    for (const QString &filename : listFiles) {
         (*cache[filename])->reparseConfiguration();
     }
 }
@@ -91,10 +96,8 @@ KNotifyConfig *KNotifyConfig::copy() const
 
 QString KNotifyConfig::readEntry(const QString &entry, bool path)
 {
-    QPair<QString, QString> context;
-
-    Q_FOREACH (context, contexts) {
-        const QString group = "Event/" + eventid + '/' + context.first + '/' + context.second;
+    for (const QPair<QString, QString> &context : qAsConst(contexts)) {
+        const QString group = QStringLiteral("Event/") + eventid + QLatin1Char('/') + context.first + QLatin1Char('/') + context.second;
 
         if (configfile->hasGroup(group)) {
             KConfigGroup cg(configfile, group);
@@ -115,7 +118,7 @@ QString KNotifyConfig::readEntry(const QString &entry, bool path)
         }
     }
 //    kDebug() << entry << " not found in contexts ";
-    const QString group = "Event/" + eventid;
+    const QString group = QStringLiteral("Event/") + eventid;
 
     if (configfile->hasGroup(group)) {
         KConfigGroup cg(configfile, group);
